searchData={"items":[{"type":"module","title":"DataTable","doc":"DataTable is a flexible and interactive table component for LiveView.\n\n```elixir\ndef render(assigns) do\n  ~H\"\"\"\n   \n\n    <:col :let={row} name=\"Id\" fields={[:id]} sort_field={:id}>\n      <%= row.id %>\n    </:col>\n\n    <:col :let={row} name=\"Name\" fields={[:first_name, :last_name]}>\n      <%= row.first_name <> \" \" <> row.last_name %>\n    </:col>\n\n   \n  \"\"\"\nend\n\ndef mount(_params, _session, socket) do\n  query = DataTable.Ecto.Query.from(\n    user in MyApp.User,\n    fields: %{\n      id: user.id,\n      first_name: user.first_name,\n      last_name: user.last_name\n    },\n    key: :id\n  )\n\n  socket = assign(socket, :source_query, query)\n\n  [...]\nend\n```","ref":"DataTable.html"},{"type":"module","title":"Common Tasks - DataTable","doc":"* [Cheat Sheet for DataTable Component](data_table_component_cheatsheet.html)\n* [Using the Ecto source](DataTable.Ecto.html)\n* [Setting up query string navigation](DataTable.NavState.html#module-persisting-datatable-state-in-query-string)","ref":"DataTable.html#module-common-tasks"},{"type":"module","title":"Getting started - DataTable","doc":"First you need to add `data_table` to your `mix.exs`:\n\n```elixir\ndefp deps do\n  [\n    {:data_table, \"~> 1.0}\n  ]\nend\n```\n\nIf you want to use the default `Tailwind` theme, you need to set up `tailwind` to include styles\nfrom the `data_table` dependency.\n\nAdd this to the `content` list in your `assets/tailwind.js`:\n```\n\"../deps/data_table/**/*.*ex\"\n```","ref":"DataTable.html#module-getting-started"},{"type":"module","title":"Data model - DataTable","doc":"Some terms you should know when using the library:\n\n* Source - A module implementing the DataTable.Source behaviour. A Source\n  provides data to the DataTable component in a pluggable way.\n  Examples of built-in sources are: DataTable.Ecto, DataTable.List\n* Data Row - A single row of data returned from the Source.\n* Data Field - A column of data returned from the Source. Example: In a\n  database table, this might be a single field like \"first_name\" or \"email\".\n* Table Column - A column displayed in the table. A Table Field may combine or\n  transform data from one or more Data Columns. Example: A \"full_name\" Table Field\n  might combine \"first_name\" and \"last_name\" Data Columns.\n\nNote: Internally, Data Fields are referred to simply as \"fields\", while Table Columns\nare called \"columns\".\n\nTo summarize, a *Source* provides *Data Fields* which are then mapped to *Table Columns*\nfor display in the *DataTable* component.","ref":"DataTable.html#module-data-model"},{"type":"function","title":"DataTable.live_data_table/1","doc":"Renders a `DataTable` in a given `LiveView` as a `LiveComponent`.\n\n`source` and `id` are required attributes.","ref":"DataTable.html#live_data_table/1"},{"type":"function","title":"Attributes - DataTable.live_data_table/1","doc":"* `theme` (`:atom`) - The theme for the DataTable. Defaults to `DataTable.Theme.Tailwind`, a modern theme\n  implemented using `tailwind`.\n\n  Defaults to `DataTable.Theme.Tailwind`.\n* `id` (`:any`) (required) - `live_data_table` is a stateful component, and requires an `id`.\n  See `LiveView.LiveComponent` for more information.\n\n* `source` (`:any`) (required) - Declares where the `DataTable` should fetch its data from.\n\n  ```\n  {source_module :: DataTable.Source.t(), source_config :: any()}\n  ```\n\n  `source_module` is a module implementing the `DataTable.Source` behaviour,\n  `source_config` is the configuration passed to the `DataTable.Source` implementation.\n\n* `nav` (`:any`) - Override the navigation state of the table.\n  Most likely only present when `handle_nav` is also present.\n\n  `nil` will be counted as no change.\n\n* `handle_nav` (`:any`) - Called when the navigation state of the table has changed.\n  If present, the navigation data should be passed back into the `nav` parameter.\n\n* `always_columns` (`:list`) - A list of column ids that will always be loaded.","ref":"DataTable.html#live_data_table/1-attributes"},{"type":"function","title":"Slots - DataTable.live_data_table/1","doc":"* `col` - One `:col` should be sepecified for each potential column in the table. Accepts attributes:\n\n  * `name` (`:string`) (required) - Name in column header. Must be unique.\n  * `visible` (`:boolean`) - Default visibility of the column.\n  * `fields` (`:list`) - List of `field`s that will be queried when this field is visible.\n  * `filter_field` (`:atom`) - If present, cells will have a filter shortcut. The filter shortcut\n    will apply a filter for the specified field.\n\n  * `filter_field_op` (`:atom`) - The filter op type which will be used for the cell filter shortcut.\n  * `sort_field` (`:atom`) - If present, columns will be sortable. The sort will occur on\n    the specified field. Defaults to the first field in `fields`.\n\n* `row_expanded` - Markup which will be rendered when a row is expanded. Accepts attributes:\n\n  * `fields` (`:list`) - List of `field`s that will be queried when a row is expanded.\n* `top_right` - Markup in the top right corner of the table.\n* `row_buttons` - Markup in the rightmost side of each row in the table. Accepts attributes:\n\n  * `fields` (`:list`) - List of `field`s that will be queried when this field is visible.\n* `selection_action` - Accepts attributes:\n\n  * `label` (`:string`) (required)\n  * `handle_action` (`:any`) (required)","ref":"DataTable.html#live_data_table/1-slots"},{"type":"module","title":"DataTable.NavState","doc":"The `NavState` struct contains the navigation state of a `DataTable`.\n\nThe `NavState` can be optionally serialized and deserialized to a query string.\n\nContains the following pieces of UI state:\n* Current page\n* Active sort\n* Active filters","ref":"DataTable.NavState.html"},{"type":"module","title":"Persisting DataTable state in query string - DataTable.NavState","doc":"We need to do 2 things:\n* 1. Decode and forward the query string to our `LiveData`\n* 2. Apply changes to `NavState` to the query string of the `LiveView`","ref":"DataTable.NavState.html#module-persisting-datatable-state-in-query-string"},{"type":"module","title":"1. Query string -> `DataTable` - DataTable.NavState","doc":"We start by implementing the `handle_params/3` callback in our `LiveView`.\n\nThis is called whenever the URI changes, and we use it to catch query\nstring changes.\n\n```elixir\ndef handle_params(_params, uri, socket) do\n  %URI{query: query} = URI.parse(uri)\n  nav = DataTable.NavState.decode_query_string(query)\n  socket = assign(socket, :nav, nav)\n  {:noreply, socket}\nend\n```\n\nThe decoded `NavState` is assigned to the `:nav` assign, which we need to\nforward to our `DataTable`.\n\n```elixir\n \n```\n\nAt this point you should be able to add a query string to your liveview\n(like `?page=5`), and see it being applied to the `DataTable` on load,\nbut the query string will not yet update on changes.","ref":"DataTable.NavState.html#module-1-query-string-datatable"},{"type":"module","title":"2. `NavState` -> query string - DataTable.NavState","doc":"The `handle_nav` callback is called whenever the nav state of the `DataTable`\nchanges. Here we use it to send a message to our LiveView.\n\n```elixir\n  send(self(), {:nav, nav}) end}/>\n```\n\nWe also need to handle `{:nav, nav}` message and push the changes to the URL.\n\n```elixir\ndef handle_info({:nav, nav}, socket) do\n  query = DataTable.NavState.encode_query_string(nav)\n  socket =\n    socket\n    |> push_patch(to: ~p\"/my/live/view\" <> query, replace: true)\n    |> assign(:nav, nav) # Important!\n  {:noreply, socket}\nend\n```\n\nNotice that we also assign the received `nav` to our `:nav` assign. This is\nimportant so that the latest state is always passed to our `DataTable`.\n\nAt this point you should be able to navigate the DataTable, see the query\nstring update, and see the changes persist on refresh.","ref":"DataTable.NavState.html#module-2-navstate-query-string"},{"type":"function","title":"DataTable.NavState.decode/2","doc":"","ref":"DataTable.NavState.html#decode/2"},{"type":"function","title":"DataTable.NavState.decode_query_string/2","doc":"","ref":"DataTable.NavState.html#decode_query_string/2"},{"type":"function","title":"DataTable.NavState.encode/1","doc":"","ref":"DataTable.NavState.html#encode/1"},{"type":"function","title":"DataTable.NavState.encode_query_string/1","doc":"","ref":"DataTable.NavState.html#encode_query_string/1"},{"type":"type","title":"DataTable.NavState.kv/0","doc":"","ref":"DataTable.NavState.html#t:kv/0"},{"type":"type","title":"DataTable.NavState.t/0","doc":"","ref":"DataTable.NavState.html#t:t/0"},{"type":"module","title":"DataTable.Ecto","doc":"This module implements a `DataTable.Source` which fetches data from\nan ecto `Repo`.\n\nYou need to pass it two arguments, a Repo and a Query:\n```elixir\n \n```\n\nThe repo is a normal `Ecto.Repo`, while the query is a query defined\nusing the DSL in `DataTable.Ecto.Query`.\n\nYou usually create your query in the `mount/3` callback of your `LiveView`:\n\n```elixir\ndef mount(_params, _session, socket) do\n  query = DataTable.Ecto.Query.from(\n    user in MyApp.User,\n    fields: %{\n      id: user.id,\n      first_name: user.first_name,\n      last_name: user.last_name\n    },\n    key: :id\n  )\n\n  socket = assign(socket, :source_query, query)\n\n  [...]\nend\n```\n\nSee `DataTable.Ecto.Query` for a full description of the query DSL.","ref":"DataTable.Ecto.html"},{"type":"module","title":"DataTable.Ecto.Query","doc":"DSL used to declare queries for use with the `DataTable.Ecto` source.\n\n```elixir\ndef mount(_params, _session, socket) do\n  query = DataTable.Ecto.Query.from(\n    user in MyApp.User,\n    fields: %{\n      id: user.id,\n      first_name: user.first_name,\n      last_name: user.last_name\n    },\n    key: :id,\n    default_order_by: user.id\n  )\n\n  socket = assign(socket, :source_query, query)\n\n  [...]\nend\n```\n\nFor a description of the differences between `Ecto.Query.from/2` and `from/2`,\nsee the docs of `from/2`.","ref":"DataTable.Ecto.Query.html"},{"type":"module","title":"On joins and complex queries - DataTable.Ecto.Query","doc":"Since `field`s are only actually requested from the Database when a column\nin the `DataTable` actually needs them, you can make your query join several\ntables and only pay the price when the columns actually are rendered.\n\nThis is very useful for admin interfaces where you want to make many pieces of\ninformation available, but not necessarily need them shown by default.\n\nAs an example, in a query like:\n```elixir\nDataTable.Ecto.Query.from(\n  article in Model.Article,\n  left_join: category in assoc(article, :category),\n  left_join: user in assoc(article, :author),\n  fields: %{\n    title: article.title,\n    body: article.body,\n    category_name: category.name,\n    author_name: author.name\n  },\n  key: :id\n)\n```\n\nAs long as the columns in your table which use `category` and `author_name` are not\nvisible, those will not be fetched by the database, and the database will likely not\neven bother doing the joins in its query plan.\n\nThe same also applies to subqueries, aggregations, etc. You should not be scared of\nincluding optional columns in your table for admin interfaces.","ref":"DataTable.Ecto.Query.html#module-on-joins-and-complex-queries"},{"type":"macro","title":"DataTable.Ecto.Query.fields/3","doc":"","ref":"DataTable.Ecto.Query.html#fields/3"},{"type":"macro","title":"DataTable.Ecto.Query.filters/2","doc":"","ref":"DataTable.Ecto.Query.html#filters/2"},{"type":"macro","title":"DataTable.Ecto.Query.from/2","doc":"Functions exactly like `Ecto.Query.from/2`, but with some minor differences:\n\n* `:select` and `:select_merge` are not accepted.\n  * `:fields` is used instead.\n* `:key` is required. `:key` should be the name of a field which uniquely\n  identifies each row.\n* `:order_by` is not accepted, as ordering is determined by the user when\n  using the table.\n  * `:default_order_by` can be used to specify a default.\n\n# Arguments\n\n## `:fields` argument\nUsed to indicate which fields are fetchable by the table.\n\nLet's compare it to `Ecto.Query.from/2`s `:select`:\n* `:fields` fetch data only when the  `DataTable` requests the field.\n  * `:select` always fetches data.\n* `:fields` can only be a map as the root.\n  * `:select` is more flexible with the structures you can return.\n\n## `:key` argument\nThe `:key` argument is always required, and is used to specify a key in\n`:fields` which uniquely identitifes the row.\n\n## `:default_order_by` argument\nSpecifies a ordering which is overridden when the `DataTable` explicitly\nsets a sort.","ref":"DataTable.Ecto.Query.html#from/2"},{"type":"macro","title":"DataTable.Ecto.Query.key/2","doc":"","ref":"DataTable.Ecto.Query.html#key/2"},{"type":"module","title":"DataTable.List","doc":"DataTable source for a native Elixir list.","ref":"DataTable.List.html"},{"type":"module","title":"DataTable.List.Config","doc":"Configuration for the List source.","ref":"DataTable.List.Config.html"},{"type":"function","title":"DataTable.List.Config.default_mapper/2","doc":"","ref":"DataTable.List.Config.html#default_mapper/2"},{"type":"behaviour","title":"DataTable.Source","doc":"","ref":"DataTable.Source.html"},{"type":"function","title":"DataTable.Source.filter_types/1","doc":"","ref":"DataTable.Source.html#filter_types/1"},{"type":"callback","title":"DataTable.Source.filter_types/1","doc":"","ref":"DataTable.Source.html#c:filter_types/1"},{"type":"function","title":"DataTable.Source.filterable_fields/1","doc":"","ref":"DataTable.Source.html#filterable_fields/1"},{"type":"callback","title":"DataTable.Source.filterable_fields/1","doc":"","ref":"DataTable.Source.html#c:filterable_fields/1"},{"type":"function","title":"DataTable.Source.key/1","doc":"","ref":"DataTable.Source.html#key/1"},{"type":"callback","title":"DataTable.Source.key/1","doc":"","ref":"DataTable.Source.html#c:key/1"},{"type":"function","title":"DataTable.Source.query/2","doc":"","ref":"DataTable.Source.html#query/2"},{"type":"callback","title":"DataTable.Source.query/2","doc":"","ref":"DataTable.Source.html#c:query/2"},{"type":"type","title":"DataTable.Source.source_module/0","doc":"","ref":"DataTable.Source.html#t:source_module/0"},{"type":"type","title":"DataTable.Source.source_opts/0","doc":"","ref":"DataTable.Source.html#t:source_opts/0"},{"type":"module","title":"DataTable.Source.Query","doc":"","ref":"DataTable.Source.Query.html"},{"type":"type","title":"DataTable.Source.Query.t/0","doc":"","ref":"DataTable.Source.Query.html#t:t/0"},{"type":"module","title":"DataTable.Source.Result","doc":"","ref":"DataTable.Source.Result.html"},{"type":"type","title":"DataTable.Source.Result.t/0","doc":"","ref":"DataTable.Source.Result.html#t:t/0"},{"type":"module","title":"DataTable.Theme.Basic","doc":"Bare minimum DataTable theme designed with two purposes in mind:\n* Serve as a bare minimum example for people wanting to develop their own\n* Simple theme for use in tests\n\nDo not expect anything pretty if you try it yourself. Not expected to\nbe very usable.","ref":"DataTable.Theme.Basic.html"},{"type":"function","title":"DataTable.Theme.Basic.checkbox/1","doc":"","ref":"DataTable.Theme.Basic.html#checkbox/1"},{"type":"function","title":"Attributes - DataTable.Theme.Basic.checkbox/1","doc":"* `state` (`:atom`)\n* `on_toggle` (`:string`) - Defaults to `nil`.\n* Global attributes are accepted.","ref":"DataTable.Theme.Basic.html#checkbox/1-attributes"},{"type":"function","title":"DataTable.Theme.Basic.root/1","doc":"","ref":"DataTable.Theme.Basic.html#root/1"},{"type":"module","title":"DataTable.Theme.Tailwind","doc":"","ref":"DataTable.Theme.Tailwind.html"},{"type":"function","title":"DataTable.Theme.Tailwind.btn_basic/1","doc":"A modern data table theme implemented using Tailwind.\n\nDesign inspired by https://www.figma.com/community/file/1021406552622495462/data-table-design-components-free-ui-kit\nby HBI Agency and Violetta Nekrasova according to CC BY 4.0.","ref":"DataTable.Theme.Tailwind.html#btn_basic/1"},{"type":"function","title":"Attributes - DataTable.Theme.Tailwind.btn_basic/1","doc":"* `size` (`:atom`) - Defaults to `:small`.","ref":"DataTable.Theme.Tailwind.html#btn_basic/1-attributes"},{"type":"function","title":"Slots - DataTable.Theme.Tailwind.btn_basic/1","doc":"* `icon`\n* `inner_block` (required)","ref":"DataTable.Theme.Tailwind.html#btn_basic/1-slots"},{"type":"function","title":"DataTable.Theme.Tailwind.btn_icon/1","doc":"","ref":"DataTable.Theme.Tailwind.html#btn_icon/1"},{"type":"function","title":"Slots - DataTable.Theme.Tailwind.btn_icon/1","doc":"* `inner_block` (required)","ref":"DataTable.Theme.Tailwind.html#btn_icon/1-slots"},{"type":"function","title":"DataTable.Theme.Tailwind.filter_header/1","doc":"","ref":"DataTable.Theme.Tailwind.html#filter_header/1"},{"type":"function","title":"DataTable.Theme.Tailwind.filters_form/1","doc":"","ref":"DataTable.Theme.Tailwind.html#filters_form/1"},{"type":"function","title":"Attributes - DataTable.Theme.Tailwind.filters_form/1","doc":"* `target` (`:any`)\n* `filters_form` (`:any`)\n* `filter_column_order` (`:any`)\n* `filter_columns` (`:any`)\n* `filters_fields` (`:any`)\n* `update_filters` (`:any`)","ref":"DataTable.Theme.Tailwind.html#filters_form/1-attributes"},{"type":"function","title":"DataTable.Theme.Tailwind.root/1","doc":"","ref":"DataTable.Theme.Tailwind.html#root/1"},{"type":"function","title":"DataTable.Theme.Tailwind.select/1","doc":"","ref":"DataTable.Theme.Tailwind.html#select/1"},{"type":"function","title":"Attributes - DataTable.Theme.Tailwind.select/1","doc":"* `field` (`Phoenix.HTML.FormField`)\n* `options` (`:any`)","ref":"DataTable.Theme.Tailwind.html#select/1-attributes"},{"type":"function","title":"DataTable.Theme.Tailwind.table_body/1","doc":"","ref":"DataTable.Theme.Tailwind.html#table_body/1"},{"type":"function","title":"DataTable.Theme.Tailwind.table_footer/1","doc":"","ref":"DataTable.Theme.Tailwind.html#table_footer/1"},{"type":"function","title":"DataTable.Theme.Tailwind.table_header/1","doc":"","ref":"DataTable.Theme.Tailwind.html#table_header/1"},{"type":"function","title":"DataTable.Theme.Tailwind.text_input/1","doc":"","ref":"DataTable.Theme.Tailwind.html#text_input/1"},{"type":"function","title":"Attributes - DataTable.Theme.Tailwind.text_input/1","doc":"* `field` (`Phoenix.HTML.FormField`)","ref":"DataTable.Theme.Tailwind.html#text_input/1-attributes"},{"type":"module","title":"DataTable.Theme.Util","doc":"Utilities which are useful for building your own theme.","ref":"DataTable.Theme.Util.html"},{"type":"function","title":"DataTable.Theme.Util.generate_pages/3","doc":"","ref":"DataTable.Theme.Util.html#generate_pages/3"},{"type":"extras","title":"DataTable Component","doc":"# DataTable Component","ref":"data_table_component_cheatsheet.html"},{"type":"extras","title":"Declaring columns - DataTable Component","doc":"#### Declaring a column\nThe `name` attribute is rendered in the header, the contents\nare rendered for each row.\n\n```elixir\n<:col name=\"Visual name of column\">\n  HTML content\n</:col>\n```\n\n#### Displaying data\nSome `Source`s (like `DataTable.Ecto`) only fetch data which is\nrequested by visible columns.\n\n```elixir\n<:col name=\"Row Id\" fields={[:id]} :let={row}>\n  <%= row.id %>\n</:col>\n```\n\n#### Sorting\nYou can enable sorting by a source field. Clicking the header will cycle sorting\nfor this field.\n\n```elixir\n<:col name=\"Id\" sort_field={:id}>\n[...]\n```\n\n#### Visibility\nColumns are visible by default. Default visibility can be changed with the `visible` attribute.\n\n```elixir\n<:col name=\"Id\" visible={false}>\n[...]\n```","ref":"data_table_component_cheatsheet.html#declaring-columns"},{"type":"extras","title":"Row decorations - DataTable Component","doc":"#### Expandable rows\n```elixir\n<:row_expanded fields={[:id, :name]} :let={row}>\n  Expanded content, hello <%= row.name %> with id <%= row.id %>\n</:row_expanded>\n```\nWhen added, rows will get an expand button.\nCan only be declared once.\n\n#### Row buttons\n```elixir\n<:row_buttons fields={[:id]} :let={row}>\n  Inner content\n</:row_buttons>\n```\nThe `row_buttons` slot is rendered all the way to the right of each row.\nCan only be declared once.","ref":"data_table_component_cheatsheet.html#row-decorations"},{"type":"extras","title":"Selections - DataTable Component","doc":"#### Selection actions\n```elixir\n<:selection_action label=\"Action name\" handle_action={callback}/>\n```\nIf specified, a selection box is added to each row.\nWhen rows are selected, a dropdown appears and allows you to trigger selection actions.","ref":"data_table_component_cheatsheet.html#selections"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}